;----------------------------------------------------------------------------
; Project: GPU Object List Example with ATARI Video Definitions
;
;
; Copyright (c) 2002 Duranik
; ALL RIGHTS RESERVED
;----------------------------------------------------------------------------
; Revision History:
; 22.02.2002 - Roland Graf	       created
; 23.10.2002 - Roland Graf	       error in OBJ_List definition corrected.
; 11.05.2003 - Roland Graf	       added CD support
; 12.05.2003 - Matthias Domin (DO) adapted to image size of 128KB, changed
;                                  track-markers and added FireB="Next Image"
;----------------------------------------------------------------------------



	.TEXT
	.68000
	.NOCLEAR


	INCLUDE "Jaguar.inc"			; ATARI Jaguar definitions
	INCLUDE	"Obj_List.inc"			; Object List definitions
	INCLUDE	"CD.inc"			; CD Bios definitions
	

	TOTAL_OBJS	EQU	2		; Anzahl Objekte


COL_LTRED       equ $f800   ; CRY= $F0FF
COL_RED         equ $7800   ; CRY= $F080
COL_LTGREEN     equ $003f   ; CRY= $7FFF
COL_GREEN       equ $001f   ; CRY= $7F80
COL_LTBLUE      equ $07c0   ; CRY= $00FF
COL_BLUE        equ $03c0   ; CRY= $0080
COL_LTCYAN      equ $07ff   ; CRY= $0FFF
COL_CYAN        equ $03df   ; CRY= $0F80
COL_LTYELLOW    equ $f83f   ; CRY= $FFFF
COL_YELLOW      equ $781f   ; CRY= $FF80
COL_LTVIOLETT   equ $ffc0   ; CRY= $F7FF
COL_VIOLETT     equ $79c0   ; CRY= $F780
COL_WHITE       equ $ffff   ; CRY= $87FF
COL_LTGRAY      equ $c630   ; CRY= $87C0
COL_GRAY        equ $8420   ; CRY= $8780
COL_BLACK       equ $0001   ; CRY= $8700



COL_TRANSPARENT equ 0


;-----------------------------------------------------------------------
; Jaguar init
;-----------------------------------------------------------------------
	move.l  #0,G_CTRL			; stop GPU
	move.l  #0,D_CTRL			; stop DSP
	move.l  #0,G_FLAGS			; init GPU Flags
	move.l  #0,D_FLAGS			; init DSP Flags
	move.l	#$00070007,G_END		; GPU Big Endian Mode
	move.l	#$00070007,D_END		; DSP Big Endian Mode
	move.w	#$35cc,MEMCON2			; Big Endian Mode
	move.l  VI,s_vi				; save old adress VI
	move.w  #$FFFF,VI			; stop Video Interrupt
	move.l  sp,s_sp				; save old Stackpointer
	move.l  #INITSTACK,sp			; init Stackpointer

	move.l	#obj_data1,obj_data


    lea.l   mypal,a0
    jsr     set_palette         ; load in the palette


    ; clear "Picture"-objectbuffer
    move.l  #Picture,a0
    move.w  #(320*240*2/4)-1,d0
    move.l  #$FFFFFFFF,d1   ; 2 white 16bit-pixels!
.clr:
    move.l  d1,(a0)+
    dbf     d0,.clr


    ; clear "score_bmp"-objectbuffer
    move.l  #score_bmp,a0
    move.w  #(320*240/2/4)-1,d0
    move.l  #0,d1   ; 8 (transparent) 4bit-pixel-indexes!
.clr2:
    move.l  d1,(a0)+
    dbf     d0,.clr2

; Print some text into the text-screen:
        move.l  #score_bmp,text_bmp ; where to copy the text

        ; White text
        move.l  #wABC,text_font
        move.b  #0,txtrow
        move.b  #8,txtcol
        move.l  #hellotext,text_adr
        bsr     do_text2

        move.b  #1,txtrow
        move.l  #author,text_adr
        bsr     do_text2



;-----------------------------------------------------------------------
; init Videosystem
;-----------------------------------------------------------------------
	bsr	VideoIni			; init Videosystem

; Create branch Objects for 2 OP-lists
	lea     olist1_branch,a0		; adress Branch Objekt
	move.l  #olist1_stop,d2			; adress Stop Objekt
	bsr     create_branch			; create Branch Objekt

	lea     olist2_branch,a0		; adress Branch Objekt
	move.l  #olist2_stop,d2			; adress Stop Objekt
	bsr     create_branch			; create Objekt erstellen

	move.l	#olist1,olist			; adresse active OP-list
	move.l	#olist2,olist_ram		; adress OP-Liste to create

	move.l	#olist1_stop,d0			; Adresse der Objekt Liste
	swap	d0				; Adresse vertauschen fr OP
	move.l	d0,OLP				; Adresse fr OP einstellen
	move.w  #$6C7,VMODE			; Video Init 320x200 16 Bit RGB


    move.l  #0,joyedge
    move.l  #0,joycur

;-----------------------------------------------------------------------
; copy adresses in GPU and program in GPU RAM
;-----------------------------------------------------------------------
	move.l  olist_ram,$f03f44		; Adresse olist_ram 
	move.l  obj_data,$f03f48		; Adresse obj_data
	move.l  #TOTAL_OBJS,$f03f4C		; Anzahl Objekte (TOTAL_OBJS)

; first screen-line
	moveq   #0,d0
	move.w  a_vdb,d0
	andi.w  #$FFFE,d0
	move.l  d0,$f03f50

	lea     gpu_prog,a0			; adress GPU routine
	bsr     copy_binfile			; copy GPU routine

	bsr     IntInit				; init Interrupts



;-----------------------------------------------------------------------
; CD-ROM init
;-----------------------------------------------------------------------
cd:	move.w	#$1865,$f00000
    nop
    nop
	move.l	#0,$dfff00			; Switch of CD-Boot-image!
    nop
    nop

; Copy the CD Bios to $3000
	move.l  #cdbios_start,a0		; CD-Bios source adress
	move.l  #(cdbios_end-cdbios_start)/4,d0	; length of CD-Bios
	move.l  #$3000,a1			; CD-Bios destination address
cd5:	
    move.l  (a0)+,(a1)+
	dbra    d0,cd5

	jsr     CD_setup			; initialize the CD system
	moveq	#%11,d0				; Single Speed, Data Mode
	jsr     CD_mode

	bsr     start_gpu			; start GPU object list build

	lea     $2C00,a0
	jsr     CD_getoc			; load CD TOC
	
	move.l  #$f03C00,a0			; load CD Bios in GPU RAM
	jsr     CD_initm



    move.l  #4,choosentrack              ; we want to load Track #4
    jsr     LoadImage


    move.l  choosentrack,d0
    subq.l  #4,d0
    addq.l  #1,d0
    add.b   #'0',d0
    move.b  d0,advice+17
    move.b  #3,txtrow
    move.b  #0,txtcol
    move.l  #advice,text_adr
    bsr     do_text2

    move.b  #4,txtrow
    move.l  #advice2,text_adr
    bsr     do_text2


;-----------------------------------------------------------------------
; Main program
;-----------------------------------------------------------------------
main:
	bsr     readpad				; Joypad abfragen
    
    move.l  joyedge,d1
ck_FireB:
    btst.l  #FIRE_B,d1          ; Firebutton B ?
    beq.s   ck_FireC            ; No

    move.l  choosentrack,d2
    add.l   #1,d2
    cmp.l   #6,d2               ; 6 = last data-track
    ble.s   .dontwrap
    move.l  #4,d2               ; 4 = first data-track
.dontwrap:
    move.l  d2,choosentrack

    jsr     LoadImage

    move.l  choosentrack,d0
    subq.l  #4,d0
    addq.l  #1,d0
    add.b   #'0',d0
    move.b  d0,advice+17
    move.b  #3,txtrow
    move.b  #0,txtcol
    move.l  #advice,text_adr
    bsr     do_text2

    move.b  #4,txtrow
    move.l  #advice2,text_adr
    bsr     do_text2


ck_FireC:

	stop	#$2000				; stop 68K until next IRQ
	bra     main




;-----------------------------------------------------------------------
; Load Image from CD-ROM
;-----------------------------------------------------------------------
; input: choosentrack=Tracknumber
;
LoadImage:
    move.l  choosentrack,d0
    asl.l   #3,d0
    lea     $2C00,a3
    add.l   d0,a3               ; Start of TOC
    
    move.l  (a3),d0
    and.l   #$00FFFFFF,d0       ; Mask off irrelevant MSB
    sub.b   #6,d0               ; Start 6 frames early
    bcc     do_read

; Handle wrap
    add.b   #75-1,d0            ; 75 frames per second
    sub.w   #$100,d0
    bcc     do_read
    add.w   #$3C00,d0           ; 256 * 60
    sub.l   #$10000,d0


do_read:
    move.l  #Picture,a0
    move.l  #Picture+(320*240*2),a1
    move.l  a1,a6               ; for later comparison
    
    move.l  #'TRAK',d1          ; partition marker
    moveq   #0,d2               ; no circular buffer
    
    jsr     CD_read
    
   
check:  
    jsr     CD_ptr
    cmp.l   a6,a0               ; all data read?
    ble     check
    jsr     CD_uread            ; yes, so stop reading
    
    ; and now stop the CD

    move.b  #6,txtrow
    move.b  #0,txtcol
    move.l  #stopcd,text_adr
    bsr     do_text2


    moveq   #1,d0               ; Wait until call completes
    jsr     CD_stop

    ;move.b  #6,txtrow
    ;move.b  #0,txtcol
    ;move.l  #stopcd,text_adr
    bsr     clear_text2


    rts


;--------------------------------------------------------------------------
; Video System initalisieren
;--------------------------------------------------------------------------
VideoIni:
	move.w  CONFIG,d0			; Also is joystick register
	andi.w  #$10,d0				; 0 = PAL, 1 = NTSC
	beq     palvals

	move.w  #NTSC_HMID,d2
	move.w  #NTSC_WIDTH,d0
	move.w  #NTSC_VMID,d6
	move.w  #NTSC_HEIGHT,d4
	bra     calc_vals

palvals:
	move.w  #PAL_HMID,d2
	move.w  #PAL_WIDTH,d0
	move.w  #PAL_VMID,d6
	move.w  #PAL_HEIGHT,d4

calc_vals:
	move.w  d0,width
	move.w  d4,height
	move.w  d0,d1
	asr     #1,d1				; Width/2
	sub.w   d1,d2				; Mid - Width/2
	add.w   #4,d2				; (Mid - Width/2)+4
	sub.w   #1,d1				; Width/2 - 1
	ori.w   #$400,d1			; (Width/2 - 1)|$400
	move.w  d1,a_hde
	move.w  d1,HDE
	move.w  d2,a_hdb
	move.w  d2,HDB1
	move.w  d2,HDB2
	move.w  d6,d5
	sub.w   d4,d5
	andi.w  #$FFFE,d5			; must be odd !
	move.w  d5,a_vdb

	add.w   d4,d6
	move.w  d6,a_vde
	move.w  a_vdb,VDB
	move.w  #$FFFF,VDE
	move.l  #0,BORD1			; Black border
	move.l  #0,BG				; Line Buffer black
	rts



;-------------------------------------------------------------------------
; Interrupts initalisieren
;-------------------------------------------------------------------------
IntInit:
	move.l	LEVEL0,s_level2
	move.l	VI,s_vi
	move.l	INT1,s_int1
	move.w	sr,s_sr

	move.l	#Frame,LEVEL0

	move.w  a_vde,d0
	ori.w   #1,d0
	move.w  d0,VI

	move.w  #1,INT1

	move.w  sr,d0
	andi.w  #$F8FF,d0
	move.w  d0,sr
	rts



;-------------------------------------------------------------------------	
; Interrupt Routine
;-------------------------------------------------------------------------	
Frame:	movem.l	d0-d1,-(sp)
	move.l	olist_ram,d0			; Adresse neue OP-Liste laden
	move.l	olist,d1			; Adresse aktive OP-Liste laden
	move.l	d1,olist_ram			; Adresse neue OP-Liste = Adresse aktive OP-Liste
	move.l	d0,olist			; Adresse aktive OP-Liste = Adresse neue OP-Liste
	
	move.l	d1,$f03f44			; Adresse olist_ram fr GPU	

	sub.l	#8*2,d0				; Adresse OP-Liste auf Branch Objekt (2 Phrases fr 2 Branchobjekte)
	swap	d0				; Adresse vertauschen fr OP
	move.l	d0,OLP				; Adresse fr OP einstellen

	move.l  obj_data,$f03f48		; Adresse obj_data
	move.l	#1,$f03f54			; start GPU object list build

	move.w	#$101,INT1
	move.w	#0,INT2
	movem.l	(sp)+,d0-d1
	rte



;--------------------------------------------------------------------------
; Branch Objekte fr Objekt Listen erstellen
;--------------------------------------------------------------------------
; In:	A0	; Adresse Branch Objekt
; 	D2	; Adresse Stop Objekt
create_branch:
; Erstes BRANCH Objekt (branch if YPOS < a_vdb)
	clr.l	d1
	move.l	#(BRANCHOBJ|O_BRGT),d0  	; YPOS < VC

; Format Link erstellen
	movem.l	d2-d3,-(sp)
	andi.l	#$3FFFF8,d2			; Ensure alignment/valid address
	move.l	d2,d3				; Make a copy

	swap	d2				; This section places bits 10-3
	clr.w	d2				; in bits 31-24. It saves cycles
	lsl.l	#5,d2				; over using three shifts.
	or.l	d2,d0

	lsr.l	#8,d3				; Put bits 21-11 in bits 42-32
	lsr.l	#3,d3
	or.l	d3,d1
	movem.l	(sp)+,d2-d3			; Restore regs
; Ende Format Link erstellen

	move.w	a_vdb,d3			; oberer Rand
	lsl.w	#3,d3				; Make it bits 13-3
	or.w	 d3,d0

	move.l	d1,(a0)+
	move.l	d0,(a0)+			; First OBJ is done.

; Zweites BRANCH Objekt (branch if YPOS > a_vde)
	andi.l	#$FF000007,d0			; Mask off CC and YPOS
	ori.l	#O_BRLT,d0			; $8000 = YPOS < VC
	move.w	a_vde,d3			; unterer Rand
	lsl.w	#3,d3				; Make it bits 13-3
	or.w	d3,d0

	move.l  d1,(a0)+			; Second OBJ is done
	move.l  d0,(a0)+
	rts



;-------------------------------------------------------------------------
; This copies the BIN file at location (a0) to its intended destination.
;-------------------------------------------------------------------------
copy_binfile:
	move.l  (a0)+,a1			; Get starting address (destination)
	move.l  (a0)+,d0			; Get length of code segment...
	asr     #2,d0				; Divide by 4 (do longs)
.loop:
    move.l  (a0)+,(a1)+
	dbra    d0,.loop
	rts



;-------------------------------------------------------------------------
; GPU starten
;-------------------------------------------------------------------------
start_gpu:
	move.l  #G_RAM,G_PC			; Start up GPU
	move.l  #1,G_CTRL
	rts




;-----------------------------------------------------------------------
; Joypad abfragen
;-----------------------------------------------------------------------
readpad:
; scan for player 1
	move.l	#$f0fffffc,d1			; d1 = Joypad data mask
	moveq.l	#-1,d2				; d2 = Cumulative joypad reading

	move.w	#$81fe,JOYSTICK
	move.l	JOYSTICK,d0			; Read joypad,pause button,A button
	or.l	d1,d0				; Mask off unused bits
	ror.l	#4,d0
	and.l	d0,d2				; d2 = xxAPxxxx RLDUxxxx xxxxxxxx xxxxxxxx
	move.w	#$81fd,JOYSTICK
	move.l	JOYSTICK,d0			; Read *741 keys,B button
	or.l	d1,d0				; Mask off unused bits
	ror.l	#8,d0
	and.l	d0,d2				; d2 = xxAPxxBx RLDU741* xxxxxxxx xxxxxxxx
	move.w	#$81fb,JOYSTICK
	move.l	JOYSTICK,d0			; Read 2580 keys,C button
	or.l	d1,d0				; Mask off unused bits
	rol.l	#6,d0
	rol.l	#6,d0
	and.l	d0,d2				; d2 = xxAPxxBx RLDU741* xxCxxxxx 2580xxxx
	move.w	#$81f7,JOYSTICK
	move.l	JOYSTICK,d0			; Read 369# keys,Option button
	or.l	d1,d0				; Mask off unused bits
	rol.l	#8,d0
	and.l	d0,d2				; d2 = xxAPxxBx RLDU741* xxCxxxOx 2580369# <== inputs active low

	moveq.l	#-1,d1
	eor.l	d2,d1				; d1 = xxAPxxBx RLDU741* xxCxxxOx 2580369# <== now inputs active high

	move.l	joycur,d0			; old joycur needed for determining the new joyedge
	move.l	d1,joycur			; Current joypad reading stored into joycur
	eor.l	d1,d0
	and.l	d1,d0
	move.l	d0,joyedge			; joypad,buttons,keys that were just pressed
	rts

;--------------------------------------------------------------------------
set_palette:        
;A0 must be the address of the palette data
;palette data is 256 words of colors
          lea.l     CLUT,a1             ;get address of color lookup table
          move.w    #255,d0             ;number of colors to set
.10:      move.w    (a0)+,(a1)+         ;put color value into color lookup table
          dbra      d0,.10              ;continue through palette table
          rts       


;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
 include "mytext.s" ; text-output, font-converting
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------



;-----------------------------------------------------------------------
; Data
;-----------------------------------------------------------------------
    .DATA



;-----------------------------------------------------------------------
; Object definitions
;-----------------------------------------------------------------------
	.QPHRASE

; Visible Object while loading CD Data
obj_data1:
		dc.w	20			; (O_XPOS)	X-Pos Object auf dem Bildschirm
		dc.w	34*2			; (O_YPOS)	Y-Pos Object auf dem Bildschirm
		dc.l	Picture			; (O_DATA)	Adresse der Object Daten
		dc.w	240			; (O_HEIGHT)	Hoehe des Objects in Pixel
		dc.w	320*2/8			; (O_IWIDTH)	Breite des Objects in Phrases
		dc.w	320*2/8			; (O_DWIDTH)	Offset zur nchsten Linie des Objects in Phrases
		dc.b	0			; (O_FLAGS)	Flags wie Object gezeichnet werden soll
		dc.b	0			; (O_FIRSTPIX)	wieviele Bits am Anfang der Object Daten uebergehen
		dc.b	0			; (O_TYPE)	Object Flags (0=Bitmap, 1=Scale, 2=GPU, 3=Branch, 4=Stop, 5=nicht aktiv)
		dc.b	4			; (O_DEPTH)	Anzahl Farben des Objects
		dc.b	1			; (O_PITCH)	Pitch
		dc.b	0			; (O_INDEX)	Index in CLUT fuer Objecte mit Farbpalette 
		dc.w	$20			; (O_SCALE)	Remainder
		dc.b	30			;		Y-Scaling Faktor
		dc.b	30			;		X-Skaling Faktor



; Textscreen:
scorex:   DC.w 20   ; o_xpos
scorey:   DC.w 34*2  ; o_ypos
scoreadr: DC.l score_bmp  ; o_data
          DC.w 240  ; o_height
          DC.w 320/2/8  ; o_dwidth
          DC.w 320/2/8  ; o_iwidth
          DC.b 4    ; o_flags   ; Transparent!!
          DC.b 0    ; o_firstpix
          DC.b 0    ; o_type    ; Bitmap (nicht skaliert)
          DC.b 2    ; o_depth   ; 16 Farben / 4Bit pro Pixel
          DC.b 1    ; o_pitch
          DC.b 0    ; o_index
          DC.w 20
          DC.b $20
          DC.b $20



    .PHRASE
mypal:
 ; Windows 16-colours palette (RGB-values):
 dc.w COL_BLACK    ; 0=Black (also the transparenzy colour!)
 ;dc.w COL_TRANSPARENT ; 0=Black (also the transparenzy colour!)

 dc.w COL_RED       ; 1=Dark Red
 dc.w COL_BLUE      ; 2=Dark Blue <------
 dc.w COL_YELLOW    ; 3=Dark Yellow
 dc.w COL_BLUE      ; 4=Dark Blue
 dc.w COL_VIOLETT   ; 5=Dark Purple
 dc.w COL_CYAN      ; 6=Dark Cyan
 dc.w COL_LTGRAY    ; 7=Light Gray
 dc.w COL_GRAY      ; 8=Dark Gray
 dc.w COL_LTRED     ; 9=Light Red
 dc.w COL_LTBLUE    ;10=Ligth Blue <------
 dc.w COL_LTYELLOW  ;11=Light Yellow
 DC.w COL_LTBLUE    ;12=Light Blue
 dc.w COL_LTVIOLETT ;13=Light Purple
 dc.w COL_LTCYAN    ;14=Light Cyan
 dc.w COL_WHITE     ;15=White


;-----------------------------------------------------------------------
; Objektliste
;-----------------------------------------------------------------------
	.QPHRASE
		rept	4
		dc.l	0			; 2 Phrases (Scaled Objekte mssen auf 4 Phrases aligned sein)
		endr
		
olist1_branch:
		rept	4
		dc.l	0			; 2 Branch Objekte (bentigen nur 2 Phrases)
		endr

olist1:
		rept	((8*TOTAL_OBJS)+2)
		dc.l	0
		endr

olist1_stop:
		dc.l	0			; 1 Teil STOP Objekt
		dc.l	4			; 2 Teil STOP Objekt


	.QPHRASE
		rept	4
		dc.l	0			; 2 Phrases (Scaled Objekte mssen auf 4 Phrases aligned sein)
		endr
olist2_branch:
		rept	4
		dc.l	0			; 2 Branch Objekte (bentigen nur 2 Phrases)
		endr

olist2:	
		rept	((8*TOTAL_OBJS)+2)
		dc.l	0
		endr

olist2_stop:
		dc.l	0			; 1 Teil STOP Objekt
		dc.l	4			; 2 Teil STOP Objekt

	

	.QPHRASE
choosentrack:   dc.l    4   ; DO
joyedge:	dc.l	0
joycur:		dc.l	0
obj_data:	dc.l	0

	.QPHRASE
gpu_prog:	incbin  "GPU_PROG.BIN"
	.QPHRASE
cdbios_start:	incbin	"cdbios45.tx"
cdbios_end:

 even
wABC: incbin "abc3.img" ; the original font


 even
hellotext:      dc.b "CD-Dataloader V2.0",0

 even
author:         dc.b "By Matthias Domin",0

 even
advice:         dc.b "This is picture #X.",0

 even
advice2:        dc.b "Press Fire B to read another picture!",0

 even
stopcd:         dc.b "Stopping the CD!",0

;-----------------------------------------------------------------------
; BSS
;-----------------------------------------------------------------------
    .BSS
    .QPHRASE

olist:      ds.l    1           ; Adresse aktive OP-Liste
olist_ram:  ds.l    1           ; Adresse zu erstellende OP-Liste
s_sp:       ds.l    1           ; Adresse alter Stackpointer
s_level2:   ds.l    1
s_vi:       ds.l    1
s_int1:     ds.l    1
s_sr:       ds.w    1
height:     ds.w    1
a_vdb:      ds.w    1
a_vde:      ds.w    1
width:      ds.w    1
a_hdb:      ds.w    1
a_hde:      ds.w    1

text_font:  ds.l 1 ; address of the actual font
text_adr:   DS.l 1 ; address of the actual text
text_bmp:   ds.l 1 ; address of the output bitmap

txtrow:     DS.b 1 ; Text row
txtcol:     DS.b 1 ; Text column

    even
wABC2:      ds.b 5760 ; Space for 96 characters (15 bitlines with 4 Bytes = 8 4bit pixels)

    .QPHRASE
score_bmp:  ds.b    320/2*240

    DPHRASE
Picture:        ds.w    320*240     ; 320*240 16bit RGB
Picture_extra:  ds.b    64


	.END

